==> openturns_0.13.1.orig.tar.gz: openturns-0.13.1/lib/src/Uncertainty/Distribution/ChiSquare.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  ChiSquare.cxx
 *  @brief The ChiSquare distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: lebrun $
 *  @date:   $LastChangedDate: 2008-11-23 16:03:50 +0100 (dim, 23 nov 2008) $
 *  Id:      $Id: ChiSquare.cxx 1026 2008-11-23 15:03:50Z lebrun $
 */
#include <cmath>
#include "ChiSquare.hxx"
#include "Gamma.hxx"
#include "RandomGenerator.hxx"
#include "SpecFunc.hxx"
#include "DistFunc.hxx"
#include "PersistentObjectFactory.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      typedef Base::Stat::RandomGenerator RandomGenerator;

      CLASSNAMEINIT(ChiSquare);

      static Base::Common::Factory<ChiSquare> RegisteredFactory("ChiSquare");

      /* Default constructor */
      ChiSquare::ChiSquare()
        : NonEllipticalDistribution("ChiSquare"),
          nu_(1.0), normalizationFactor_(1.0)
      {
        setDimension(1);
        computeRange();
      }

      /* Parameters constructor */
      ChiSquare::ChiSquare(const NumericalScalar nu)
        throw (InvalidArgumentException)
        : NonEllipticalDistribution("ChiSquare"),
          nu_(0.0), normalizationFactor_(0.0)
      {
        setNu(nu);
        setDimension(1);
        computeRange();
        update();
      }

      /* Comparison operator */
      Bool ChiSquare::operator ==(const ChiSquare & other) const
      {
        Bool sameObject = false;

        if (this != &other) { // Other is NOT me, so I have to realize the comparison
          // sameObject = ...
          // TODO: Write ChiSquare::operator ==(...)
          if (nu_ == other.nu_)
            sameObject = true;
        } else sameObject = true;

        return sameObject;
      }

      /* String converter */
      String ChiSquare::__repr__() const
      {
        OSS oss;
        oss << "class=" << ChiSquare::GetClassName()
            << " name=" << getName()
            << " dimension=" << getDimension()
            << " nu=" << nu_;
        return oss;
      }

      /* Nu accessor */
      void ChiSquare::setNu(const NumericalScalar nu)
        throw(InvalidArgumentException)
      {
        if (nu <= 0.0) throw InvalidArgumentException(HERE) << "Nu MUST be positive";
        nu_ = nu;
        update();
      }

      NumericalScalar ChiSquare::getNu() const
      {
        return nu_;
      }


      /* Virtual constructor */
      ChiSquare * ChiSquare::clone() const
      {
        return new ChiSquare(*this);
      }

      /* Compute the numerical range of the distribution given the parameters values */
      void ChiSquare::computeRange()
      {
        setRange(Gamma(0.5, 0.5 * nu_, 0.0).getRange());
      }

      /** Update the derivative attributes */
      void ChiSquare::update()
      {
        normalizationFactor_ = pow(0.5, 0.5 * nu_) / SpecFunc::Gamma(0.5 * nu_);
      }


      /* Get one realization of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getRealization() const
      {
        return NumericalPoint(1, 2.0 * DistFunc::rGamma(0.5 * nu_));
      }


      /* Get the DDF of the distribution */
      ChiSquare::NumericalPoint ChiSquare::computeDDF(const NumericalPoint & point) const
      {
        NumericalScalar x(point[0]);
        if (x <= 0.0) return NumericalPoint(1, 0.0);
        return NumericalPoint(1, ((0.5 * nu_ - 1.0) / x - 0.5) * computePDF(point));
      }


      /* Get the PDF of the distribution */
      NumericalScalar ChiSquare::computePDF(const NumericalPoint & point) const
      {
        NumericalScalar x(point[0]);
        if (x <= 0.0) return 0.0;
        return normalizationFactor_ * pow(x, 0.5 * nu_ - 1) * exp(-0.5 * x);
      }


      /* Get the CDF of the distribution */
      NumericalScalar ChiSquare::computeCDF(const NumericalPoint & point, const Bool tail) const
      {
        NumericalScalar x(point[0]);
        // No test here as the CDF is continuous for all nu_
        if (x <= 0.0) return (tail ? 1.0 : 0.0);
        return DistFunc::pGamma(0.5 * nu_, 0.5 * x, tail);
      }

      /* Get the characteristic function of the distribution, i.e. phi(u) = E(exp(I*u*X)) */
      NumericalComplex ChiSquare::computeCharacteristicFunction(const NumericalScalar x,
                                                                const Bool logScale) const
      {
        if (logScale) return -0.5 * nu_ * log(NumericalComplex(1.0, -2.0 * x));
        return pow(NumericalComplex(1.0, -2.0 * x), -0.5 * nu_);
      }

      /* Get the PDFGradient of the distribution */
      ChiSquare::NumericalPoint ChiSquare::computePDFGradient(const NumericalPoint & point) const
      {
        NumericalPoint pdfGradient(1, 0.0);
        NumericalScalar x(point[0]);
        if (x <= 0.0) return pdfGradient;
        NumericalScalar pdf(computePDF(point));
        pdfGradient[0] = 0.5 * (log(0.5 * x) - SpecFunc::Psi(0.5 * nu_)) * pdf;
        return pdfGradient;
      }

      /* Get the CDFGradient of the distribution */
      ChiSquare::NumericalPoint ChiSquare::computeCDFGradient(const NumericalPoint & point) const
      {
        NumericalPoint cdfGradient(1, 0.0);
        NumericalScalar x(point[0]);
        if (x <= 0.0) return cdfGradient;
        NumericalScalar eps(pow(DistFunc::Precision, 1.0 / 3.0));
        cdfGradient[0] = (DistFunc::pGamma(0.5 * (nu_ + eps), 0.5 * x) - DistFunc::pGamma(0.5 * (nu_ - eps), 0.5 * x)) / (2.0 * eps);
        return cdfGradient;
      }

      /* Get the quantile of the distribution */
      NumericalScalar ChiSquare::computeScalarQuantile(const NumericalScalar prob,
                                                       const NumericalScalar initialGuess,
                                                       const NumericalScalar initialStep,
                                                       const NumericalScalar precision) const
      {
        return 2.0 * DistFunc::qGamma(0.5 * nu_, prob);
      }

      /* Get the mean of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getMean() const throw(NotDefinedException)
      {
        return NumericalPoint(1, nu_);
      }

      /* Get the standard deviation of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getStandardDeviation() const throw(NotDefinedException)
      {
        return NumericalPoint(1, sqrt(2.0 * nu_));
      }

      /* Get the skewness of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getSkewness() const throw(NotDefinedException)
      {
        return NumericalPoint(1, sqrt(8.0 / nu_));
      }

      /* Get the kurtosis of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getKurtosis() const throw(NotDefinedException)
      {
        return NumericalPoint(1, 3.0 + 12.0 / nu_);
      }

      /* Get the covariance of the distribution */
      ChiSquare::CovarianceMatrix ChiSquare::getCovariance() const throw(NotDefinedException)
      {
        CovarianceMatrix covariance(1);
        covariance(0, 0) = 2.0 * nu_;
        return covariance;
      }

      /* Parameters value and description accessor */
      ChiSquare::NumericalPointWithDescriptionCollection ChiSquare::getParametersCollection() const
      {
        NumericalPointWithDescriptionCollection parameters(1);
        NumericalPointWithDescription point(1);
        Description description(point.getDimension());
        point[0] = nu_;
        description[0] = "nu";
        point.setDescription(description);
        point.setName(getDescription()[0]);
        parameters[0] = point;
        return parameters;
      }

      void ChiSquare::setParametersCollection(const NumericalPointCollection & parametersCollection)
      {
        *this = ChiSquare(parametersCollection[0][0]);
      }

      /* Method save() stores the object through the StorageManager */
      void ChiSquare::save(const StorageManager::Advocate & adv) const
      {
        NonEllipticalDistribution::save(adv);
        adv.writeValue("nu_", nu_);
        adv.writeValue("normalizationFactor_", normalizationFactor_);
      }

      /* Method load() reloads the object from the StorageManager */
      void ChiSquare::load(const StorageManager::Advocate & adv)
      {
        NonEllipticalDistribution::load(adv);

        String name;
        NumericalScalar value;
        StorageManager::List objList = adv.getList(StorageManager::NumericalScalarEntity);
        for(objList.firstValueToRead(); objList.moreValuesToRead(); objList.nextValueToRead()) {
          if (objList.readValue(name, value)) {
            if (name == "nu_") nu_ = value;
            if (name == "normalizationFactor_") normalizationFactor_ = value;
          }
        }
        computeRange();
      }




    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

==> openturns_0.13.2.orig.tar.gz: openturns-0.13.2/lib/src/Uncertainty/Distribution/ChiSquare.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  ChiSquare.cxx
 *  @brief The ChiSquare distribution
 *
 *  (C) Copyright 2005-2010 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: lebrun $
 *  @date:   $LastChangedDate: 2008-11-23 16:03:50 +0100 (dim, 23 nov 2008) $
 *  Id:      $Id: ChiSquare.cxx 1026 2008-11-23 15:03:50Z lebrun $
 */
#include <cmath>
#include "ChiSquare.hxx"
#include "Gamma.hxx"
#include "RandomGenerator.hxx"
#include "SpecFunc.hxx"
#include "DistFunc.hxx"
#include "PersistentObjectFactory.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      typedef Base::Stat::RandomGenerator RandomGenerator;

      CLASSNAMEINIT(ChiSquare);

      static Base::Common::Factory<ChiSquare> RegisteredFactory("ChiSquare");

      /* Default constructor */
      ChiSquare::ChiSquare()
        : NonEllipticalDistribution("ChiSquare"),
          nu_(1.0), normalizationFactor_(1.0)
      {
        setDimension(1);
        computeRange();
      }

      /* Parameters constructor */
      ChiSquare::ChiSquare(const NumericalScalar nu)
        /* throw (InvalidArgumentException) */
        : NonEllipticalDistribution("ChiSquare"),
          nu_(0.0), normalizationFactor_(0.0)
      {
        setNu(nu);
        setDimension(1);
        computeRange();
        update();
      }

      /* Comparison operator */
      Bool ChiSquare::operator ==(const ChiSquare & other) const
      {
        Bool sameObject = false;

        if (this != &other) { // Other is NOT me, so I have to realize the comparison
          // sameObject = ...
          // TODO: Write ChiSquare::operator ==(...)
          if (nu_ == other.nu_)
            sameObject = true;
        } else sameObject = true;

        return sameObject;
      }

      /* String converter */
      String ChiSquare::__repr__() const
      {
        OSS oss;
        oss << "class=" << ChiSquare::GetClassName()
            << " name=" << getName()
            << " dimension=" << getDimension()
            << " nu=" << nu_;
        return oss;
      }

      /* Nu accessor */
      void ChiSquare::setNu(const NumericalScalar nu)
      /* throw(InvalidArgumentException) */
      {
        if (nu <= 0.0) throw InvalidArgumentException(HERE) << "Nu MUST be positive";
        nu_ = nu;
        update();
      }

      NumericalScalar ChiSquare::getNu() const
      {
        return nu_;
      }


      /* Virtual constructor */
      ChiSquare * ChiSquare::clone() const
      {
        return new ChiSquare(*this);
      }

      /* Compute the numerical range of the distribution given the parameters values */
      void ChiSquare::computeRange()
      {
        setRange(Gamma(0.5, 0.5 * nu_, 0.0).getRange());
      }

      /** Update the derivative attributes */
      void ChiSquare::update()
      {
        normalizationFactor_ = pow(0.5, 0.5 * nu_) / SpecFunc::Gamma(0.5 * nu_);
      }


      /* Get one realization of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getRealization() const
      {
        return NumericalPoint(1, 2.0 * DistFunc::rGamma(0.5 * nu_));
      }


      /* Get the DDF of the distribution */
      ChiSquare::NumericalPoint ChiSquare::computeDDF(const NumericalPoint & point) const
      {
        NumericalScalar x(point[0]);
        if (x <= 0.0) return NumericalPoint(1, 0.0);
        return NumericalPoint(1, ((0.5 * nu_ - 1.0) / x - 0.5) * computePDF(point));
      }


      /* Get the PDF of the distribution */
      NumericalScalar ChiSquare::computePDF(const NumericalPoint & point) const
      {
        NumericalScalar x(point[0]);
        if (x <= 0.0) return 0.0;
        return normalizationFactor_ * pow(x, 0.5 * nu_ - 1) * exp(-0.5 * x);
      }


      /* Get the CDF of the distribution */
      NumericalScalar ChiSquare::computeCDF(const NumericalPoint & point,
                                            const Bool tail) const
      {
        NumericalScalar x(point[0]);
        // No test here as the CDF is continuous for all nu_
        if (x <= 0.0) return (tail ? 1.0 : 0.0);
        return DistFunc::pGamma(0.5 * nu_, 0.5 * x, tail);
      }

      /* Get the characteristic function of the distribution, i.e. phi(u) = E(exp(I*u*X)) */
      NumericalComplex ChiSquare::computeCharacteristicFunction(const NumericalScalar x,
                                                                const Bool logScale) const
      {
        if (logScale) return -0.5 * nu_ * log(NumericalComplex(1.0, -2.0 * x));
        return pow(NumericalComplex(1.0, -2.0 * x), -0.5 * nu_);
      }

      /* Get the PDFGradient of the distribution */
      ChiSquare::NumericalPoint ChiSquare::computePDFGradient(const NumericalPoint & point) const
      {
        NumericalPoint pdfGradient(1, 0.0);
        NumericalScalar x(point[0]);
        if (x <= 0.0) return pdfGradient;
        NumericalScalar pdf(computePDF(point));
        pdfGradient[0] = 0.5 * (log(0.5 * x) - SpecFunc::Psi(0.5 * nu_)) * pdf;
        return pdfGradient;
      }

      /* Get the CDFGradient of the distribution */
      ChiSquare::NumericalPoint ChiSquare::computeCDFGradient(const NumericalPoint & point) const
      {
        NumericalPoint cdfGradient(1, 0.0);
        NumericalScalar x(point[0]);
        if (x <= 0.0) return cdfGradient;
        NumericalScalar eps(pow(DistFunc::Precision, 1.0 / 3.0));
        cdfGradient[0] = (DistFunc::pGamma(0.5 * (nu_ + eps), 0.5 * x) - DistFunc::pGamma(0.5 * (nu_ - eps), 0.5 * x)) / (2.0 * eps);
        return cdfGradient;
      }

      /* Get the quantile of the distribution */
      NumericalScalar ChiSquare::computeScalarQuantile(const NumericalScalar prob,
                                                       const Bool tail,
                                                       const NumericalScalar precision) const
      {
        return 2.0 * DistFunc::qGamma(0.5 * nu_, prob, tail);
      }

      /* Get the mean of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getMean() const /* throw(NotDefinedException) */
      {
        return NumericalPoint(1, nu_);
      }

      /* Get the standard deviation of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getStandardDeviation() const /* throw(NotDefinedException) */
      {
        return NumericalPoint(1, sqrt(2.0 * nu_));
      }

      /* Get the skewness of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getSkewness() const /* throw(NotDefinedException) */
      {
        return NumericalPoint(1, sqrt(8.0 / nu_));
      }

      /* Get the kurtosis of the distribution */
      ChiSquare::NumericalPoint ChiSquare::getKurtosis() const /* throw(NotDefinedException) */
      {
        return NumericalPoint(1, 3.0 + 12.0 / nu_);
      }

      /* Get the covariance of the distribution */
      ChiSquare::CovarianceMatrix ChiSquare::getCovariance() const /* throw(NotDefinedException) */
      {
        CovarianceMatrix covariance(1);
        covariance(0, 0) = 2.0 * nu_;
        return covariance;
      }

      /* Get the moments of the standardized distribution */
      ChiSquare::NumericalPoint ChiSquare::getStandardMoment(const UnsignedLong n) const
      {
        return NumericalPoint(1, exp(n * M_LN2 + SpecFunc::LnGamma(n + 0.5 * nu_) - SpecFunc::LnGamma(0.5 * nu_)));
      }

      /* Parameters value and description accessor */
      ChiSquare::NumericalPointWithDescriptionCollection ChiSquare::getParametersCollection() const
      {
        NumericalPointWithDescriptionCollection parameters(1);
        NumericalPointWithDescription point(1);
        Description description(point.getDimension());
        point[0] = nu_;
        description[0] = "nu";
        point.setDescription(description);
        point.setName(getDescription()[0]);
        parameters[0] = point;
        return parameters;
      }

      void ChiSquare::setParametersCollection(const NumericalPointCollection & parametersCollection)
      {
        *this = ChiSquare(parametersCollection[0][0]);
      }

      /* Method save() stores the object through the StorageManager */
      void ChiSquare::save(StorageManager::Advocate & adv) const
      {
        NonEllipticalDistribution::save(adv);
        adv.saveAttribute( "nu_", nu_ );
        adv.saveAttribute( "normalizationFactor_", normalizationFactor_ );
      }

      /* Method load() reloads the object from the StorageManager */
      void ChiSquare::load(StorageManager::Advocate & adv)
      {
        NonEllipticalDistribution::load(adv);
        adv.loadAttribute( "nu_", nu_ );
        adv.loadAttribute( "normalizationFactor_", normalizationFactor_ );
        computeRange();
      }




    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */
